From db04f0795a6af53a1a8c7437299c2c9da29cdeee Mon Sep 17 00:00:00 2001
From: "Chris West (Faux)" <git@goeswhere.com>
Date: Sun, 24 Jan 2016 20:22:57 +0000
Subject: [PATCH] non-mono pe files just go straight through objdump -D

---
 diffoscope/comparators/__init__.py |  2 ++
 diffoscope/comparators/pe.py       | 41 ++++++++++++++++++++++++++++++++++++++
 2 files changed, 43 insertions(+)
 create mode 100644 diffoscope/comparators/pe.py

diff --git a/diffoscope/comparators/__init__.py b/diffoscope/comparators/__init__.py
index 5224ca1..c02e827 100644
--- a/diffoscope/comparators/__init__.py
+++ b/diffoscope/comparators/__init__.py
@@ -58,6 +58,7 @@ from diffoscope.comparators.iso9660 import Iso9660File
 from diffoscope.comparators.macho import MachoFile
 from diffoscope.comparators.mono import MonoExeFile
 from diffoscope.comparators.pdf import PdfFile
+from diffoscope.comparators.pe import PeFile
 from diffoscope.comparators.png import PngFile
 from diffoscope.comparators.ppu import PpuFile
 try:
@@ -149,6 +150,7 @@ FILE_CLASSES = (
     Iso9660File,
     ClassFile,
     MonoExeFile,
+    PeFile, # the mono comparator is more complete, and stricter at guessing
     PdfFile,
     PngFile,
     PpuFile,
diff --git a/diffoscope/comparators/pe.py b/diffoscope/comparators/pe.py
new file mode 100644
index 0000000..c3ee92c
--- /dev/null
+++ b/diffoscope/comparators/pe.py
@@ -0,0 +1,41 @@
+# -*- coding: utf-8 -*-
+#
+# diffoscope: in-depth comparison of files, archives, and directories
+#
+# diffoscope is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# diffoscope is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with diffoscope.  If not, see <http://www.gnu.org/licenses/>.
+
+import re
+from diffoscope import tool_required
+from diffoscope.comparators.binary import File
+from diffoscope.comparators.utils import Command
+from diffoscope.difference import Difference
+
+
+class ObjdumpDisassembleAll(Command):
+    @tool_required('objdump')
+    def cmdline(self):
+        return ['objdump', '-D', self.path]
+
+
+class PeFile(File):
+    RE_FILE_TYPE = re.compile(r'\bPE[0-9]+ executable\b.*\bIntel\b.*, for MS Windows')
+
+    @staticmethod
+    def recognizes(file):
+        return PeFile.RE_FILE_TYPE.search(file.magic_file_type)
+
+    def compare_details(self, other, source=None):
+        return [
+            Difference.from_command(ObjdumpDisassembleAll, self.path, other.path),
+        ]
-- 
2.5.0

